def main() -> None:
    import functools

    import frida

    from frida_tools.application import ConsoleApplication

    class LSDApplication(ConsoleApplication):
        def _usage(self) -> str:
            return "%(prog)s [options]"

        def _needs_device(self) -> bool:
            return False

        def _start(self) -> None:
            try:
                devices = frida.enumerate_devices()
            except Exception as e:
                self._update_status(f"Failed to enumerate devices: {e}")
                self._exit(1)
                return
            device_name = {}
            device_os = {}
            for device in devices:
                device_name[device.id] = device.name
                try:
                    params = device.query_system_parameters()
                except:
                    continue
                device_name[device.id] = params.get("name", device.name)
                os = params["os"]
                version = os.get("version")
                if version is not None:
                    device_os[device.id] = os["name"] + " " + version
                else:
                    device_os[device.id] = os["name"]
            id_column_width = max(map(lambda device: len(device.id) if device.id is not None else 0, devices))
            type_column_width = max(map(lambda device: len(device.type) if device.type is not None else 0, devices))
            name_column_width = max(map(lambda name: len(name) if name is not None else 0, device_name.values()))
            os_column_width = max(map(lambda os: len(os) if os is not None else 0, device_os.values()))
            header_format = (
                "%-"
                + str(id_column_width)
                + "s  "
                + "%-"
                + str(type_column_width)
                + "s  "
                + "%-"
                + str(name_column_width)
                + "s  "
                + "%-"
                + str(os_column_width)
                + "s"
            )
            self._print(header_format % ("Id", "Type", "Name", "OS"))
            self._print(
                f"{id_column_width * '-'}  {type_column_width * '-'}  {name_column_width * '-'}  {os_column_width * '-'}"
            )
            line_format = (
                "%-"
                + str(id_column_width)
                + "s  "
                + "%-"
                + str(type_column_width)
                + "s  "
                + "%-"
                + str(name_column_width)
                + "s  "
                + "%-"
                + str(os_column_width)
                + "s"
            )
            for device in sorted(devices, key=functools.cmp_to_key(compare_devices)):
                self._print(
                    line_format % (device.id, device.type, device_name.get(device.id), device_os.get(device.id, ""))
                )
            self._exit(0)

    def compare_devices(a: frida.core.Device, b: frida.core.Device) -> int:
        a_score = score(a)
        b_score = score(b)
        if a_score == b_score:
            if a.name is None or b.name is None:
                return 0
            if a.name > b.name:
                return 1
            elif a.name < b.name:
                return -1
            else:
                return 0
        else:
            if a_score > b_score:
                return -1
            elif a_score < b_score:
                return 1
            else:
                return 0

    def score(device: frida.core.Device) -> int:
        type = device.type
        if type == "local":
            return 3
        elif type == "usb":
            return 2
        else:
            return 1

    app = LSDApplication()
    app.run()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



from __future__ import annotations

import sys
from typing import TextIO, cast

from prompt_toolkit.utils import (
    get_bell_environment_variable,
    get_term_environment_variable,
    is_conemu_ansi,
)

from .base import DummyOutput, Output
from .color_depth import ColorDepth
from .plain_text import PlainTextOutput

__all__ = [
    "create_output",
]


def create_output(
    stdout: TextIO | None = None, always_prefer_tty: bool = False
) -> Output:
    """
    Return an :class:`~prompt_toolkit.output.Output` instance for the command
    line.

    :param stdout: The stdout object
    :param always_prefer_tty: When set, look for `sys.stderr` if `sys.stdout`
        is not a TTY. Useful if `sys.stdout` is redirected to a file, but we
        still want user input and output on the terminal.

        By default, this is `False`. If `sys.stdout` is not a terminal (maybe
        it's redirected to a file), then a `PlainTextOutput` will be returned.
        That way, tools like `print_formatted_text` will write plain text into
        that file.
    """
    # Consider TERM, PROMPT_TOOLKIT_BELL, and PROMPT_TOOLKIT_COLOR_DEPTH
    # environment variables. Notice that PROMPT_TOOLKIT_COLOR_DEPTH value is
    # the default that's used if the Application doesn't override it.
    term_from_env = get_term_environment_variable()
    bell_from_env = get_bell_environment_variable()
    color_depth_from_env = ColorDepth.from_env()

    if stdout is None:
        # By default, render to stdout. If the output is piped somewhere else,
        # render to stderr.
        stdout = sys.stdout

        if always_prefer_tty:
            for io in [sys.stdout, sys.stderr]:
                if io is not None and io.isatty():
                    # (This is `None` when using `pythonw.exe` on Windows.)
                    stdout = io
                    break

    # If the output is still `None`, use a DummyOutput.
    # This happens for instance on Windows, when running the application under
    # `pythonw.exe`. In that case, there won't be a terminal Window, and
    # stdin/stdout/stderr are `None`.
    if stdout is None:
        return DummyOutput()

    # If the patch_stdout context manager has been used, then sys.stdout is
    # replaced by this proxy. For prompt_toolkit applications, we want to use
    # the real stdout.
    from prompt_toolkit.patch_stdout import StdoutProxy

    while isinstance(stdout, StdoutProxy):
        stdout = stdout.original_stdout

    if sys.platform == "win32":
        from .conemu import ConEmuOutput
        from .win32 import Win32Output
        from .windows10 import Windows10_Output, is_win_vt100_enabled

        if is_win_vt100_enabled():
            return cast(
                Output,
                Windows10_Output(stdout, default_color_depth=color_depth_from_env),
            )
        if is_conemu_ansi():
            return cast(
                Output, ConEmuOutput(stdout, default_color_depth=color_depth_from_env)
            )
        else:
            return Win32Output(stdout, default_color_depth=color_depth_from_env)
    else:
        from .vt100 import Vt100_Output

        # Stdout is not a TTY? Render as plain text.
        # This is mostly useful if stdout is redirected to a file, and
        # `print_formatted_text` is used.
        if not stdout.isatty():
            return PlainTextOutput(stdout)

        return Vt100_Output.from_pty(
            stdout,
            term=term_from_env,
            default_color_depth=color_depth_from_env,
            enable_bell=bell_from_env,
        )
